@@ -4,45 +4,24 @@ use strict;
 use warnings;
 use base 'Module::Info';
 use Carp;
-use File::Spec;
+use File::Basename; # fileparse
 use vars qw($VERSION);
+use Module::Metadata;
+use Try::Tiny; # try catch
 
-$VERSION = '0.14';
+$VERSION = '1.00';
 
 sub new_from_file {
-    my ( $proto, $filename ) = @_;
+    my ( $proto, $filepath ) = @_;
 
-    open my $file, '<', $filename
-        or croak "Unable to open file: $filename - $!";
-
-    my ( @packages, $version, $name, $dir );
-    while (<$file>) {
-        if ( $_ =~ m/\bpackage\s?([A-Za-z0-9_:]+);/xm ) {
-            if ($1) {
-                $name = $1;
-            } else {
-                ($name) = $file =~ m/(\w+\.pm)$/xm;
-            }
-        }
-        $dir = File::Spec->rel2abs($file);
-        $dir =~ s[/(\w+\.p(m|l))$][]xm;
-
-        if ( $_ =~ m/\bVERSION\b\s*=\s*'([0-9_.]+)'*;/xm ) {
-            $version = $1;
-        }
-
-        if ( $name and $dir and $filename and $version ) {
-            last;
-        }
-
-    }
-    close $file;
+    my $info = Module::Metadata->new_from_file($filepath);
+    my ($filename, $dir, $suffix) = fileparse($filepath);
 
     return __PACKAGE__->new_from_data(
-        name    => $name,
+        name    => $info->name,
         dir     => $dir,
         file    => $filename,
-        version => $version ? $version : undef,
+        version => $info->version,
     );
 }
 
@@ -72,8 +51,19 @@ sub version {
 
     if ( $self->{version} ) {
         return $self->{version};
-    } else {
-        return $self->SUPER::version();
+    }
+    else {
+        try {
+            my $version = $self->SUPER::version();
+
+            if ($version) {
+                return $version;
+            } else {
+                return undef;
+            }
+        } catch {
+            return undef;
+        };
     }
 }
 
@@ -97,7 +87,7 @@ Module::Info::File - retrieve module information from a file
 
 =head1 VERSION
 
-This POD describes version 0.13 of Module::Info::File
+This POD describes version 1.00 of Module::Info::File
 
 =head1 SYNOPSIS
 
@@ -105,25 +95,37 @@ This POD describes version 0.13 of Module::Info::File
 
 	my $module = Module::Info::File->new_from_file('path/to/Some/Module.pm');
 
-	$mod->name();
+	$module->name();
 
-	$mod->version();
+	$module->version();
 
-	$mod->file();
+	$module->file();
 
-	$mod->inc_dir();
+	$module->inc_dir();
 
 =head1 DESCRIPTION
 
-Module::Info (SEE REFERENCES), are lacking functionality of being able
+B<Module::Info> (SEE REFERENCES), are lacking functionality of being able
 to extract certain data when parsing a module directly from a file. I
 have therefor created Module::Info::File, which inherits from
 Module::Info and replaces the B<new_from_file> method so the lacking
 data can be accessed (dir and name attributes). Apart from that you can
-use all the neat accessors from Module::Info.
+use all the neat accessors from B<Module::Info>.
 
-In the bin folder in this distribution is a small script called
-version.pl, which was the beginning of everything.
+Given the following structure in a file:
+
+    package Foo;
+    1;
+    package Bar;
+    1;
+
+B<Module::Info::File> returns: Foo
+
+Given the following structure in a file:
+
+    package Foo;
+    $VERSION = '0.01';
+    1;
 
 =head1 SUBROUTINES/METHODS
 
@@ -148,8 +150,8 @@ the object:
 So by using the inherited methods from B<Module::Info> you can access
 the attributes.
 
-There is an example in the bin/ folder called version.pl, this script
-was the starting point for this module.
+In the F<script> folder in this distribution is a small script called
+F<version.pl>, which was the beginning of this module.
 
 =over 4
 
@@ -168,7 +170,6 @@ was the starting point for this module.
 
 A helper method to streamline the result set
 
-
 In general please refer to the documentation on B<Module::Info> for more
 details.
 
@@ -187,7 +188,7 @@ Module::Info::File will be obsolete and can be discontinued.
 
 =item * Module::Info, by Mattia Barbon
 
-=item * bin/version.pl
+=item * F<script/version.pl>
 
 =back
 
@@ -225,7 +226,7 @@ for further information on this issue, regarding his version and installation.
 
 =over
 
-=item * Unable to open file: <filename> - <operating system error>
+=item * C<< Unable to open file: <filename> - <operating system error> >>
 
 If the constructor B<new_from_file> is given a filename parameter, which does
 not meet the following prerequisites:
@@ -263,15 +264,6 @@ require this I would be willing to invest the time and effort.
 
 =back
 
-=head1 TODO
-
-=over
-
-=item * Use PPI for parsing instead of own parsing mechanisms or it this serious
-overkill?
-
-=back
-
 =head1 ACKNOWLEDGEMENTS
 
 In no particular order
@@ -280,6 +272,10 @@ In no particular order
 
 =item *
 
+Dai Okabayashi (bayashi), for PRs #2 and #3
+
+=item *
+
 Jeffrey Ryan Thalhammer, for L<Perl::Critic>
 
 =item *
@@ -323,7 +319,7 @@ jonasbn E<lt>jonasbn@cpan.orgE<gt>
 Module::Info::File and related modules are free software and is
 released under the Artistic License 2.0.
 
-Module::Info::File is (C) 2003-2014 Jonas B. Nielsen (jonasbn)
+Module::Info::File is (C) 2003-2017 Jonas B. Nielsen (jonasbn)
 E<lt>jonasbn@cpan.orgE<gt>
 
 =cut
